import 'dart:async';
import 'dart:developer';

import 'package:equatable/equatable.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:soliq_id/src/data/dio/id_repository.dart';
import 'package:soliq_id/src/data/model/response/account_info_model.dart';
import 'package:soliq_id/src/data/model/response/compare_face_response.dart';
import 'package:soliq_id/src/data/model/response/create_session.dart';
import 'package:soliq_id/src/data/model/response/process_response_model.dart';

part 'soliq_id_event.dart';
part 'soliq_id_state.dart';

class SoliqIdBloc extends Bloc<SoliqIdEvent, SoliqIdState> {
  // SoliqIdRepository repository;
  IdRepository idRepository;

  SoliqIdBloc(this.idRepository) : super(SoliqIdInitial()) {
    on<SoliqIdAccountInfoEvent>(_onAccountInfo);
    on<SoliqIdCreateSessionEvent>(_onCreateSession);
    on<SoliqIdProcessEvent>(_onSetProcess);
    on<SoliqIdCompareFaceEvent>(_onSetCompareFace);
  }

  FutureOr<void> _onAccountInfo(
      SoliqIdAccountInfoEvent event, Emitter<SoliqIdState> emit) async {
    // try {
    // final response = await repository.setAccountInfo(
    //   pinfl: event.pinfl,
    //   lang: event.lang,
    // );

    emit(SoliqIdPinflLoadingState());
    final response = await idRepository.setAccountInfo(
      pinfl: event.pinfl,
    );

    // print(Utils.getDeviceDetails().toString());
    if (response.status && response.data != null) {
      emit(SoliqIdAccountInfoSuccessState(
        response: response.data!,
      ));
    } else {
      emit(SoliqIdErrorState(
        error: response.responseMessage!,
      ));
    }
    // } on Object catch (e) {
    //   log(e.toString());
    // }
  }

  FutureOr<void> _onCreateSession(
      SoliqIdCreateSessionEvent event, Emitter<SoliqIdState> emit) async {
    emit(SoliqIdPinflLoadingState());
    try {
      // final response = await repository.createSession(
      //   pinfl: event.pinfl,
      //   lang: event.lang,
      //   fio: event.fio,
      // );
      // emit(SoliqIdCreateSessionSuccessState(
      //   response: response,
      // ));

      final response = await idRepository.createSession(
        pinfl: event.pinfl,
        fio: event.fio,
      );

      if (response.status && response.data != null) {
        emit(SoliqIdCreateSessionSuccessState(
          response: response.data!,
        ));
      } else {
        emit(SoliqIdErrorState(
          error: response.responseMessage!,
        ));
      }
    } on Object catch (e) {
      log(e.toString());
    }
  }

  FutureOr<void> _onSetProcess(
      SoliqIdProcessEvent event, Emitter<SoliqIdState> emit) async {
    emit(SoliqIdPinflLoadingState());
    try {
      // final response = await repository.setProcess(
      //   id: event.id,
      //   asked: event.asked,
      //   processId: event.processId,
      //   date: '',
      // );
      // emit(SoliqIdProcessSuccessState(
      //   response: response,
      // ));

      final response = await idRepository.setProcess(
        id: event.id,
        asked: event.asked,
        processId: event.processId,
        date: '',
      );

      if (response.status && response.data != null) {
        emit(SoliqIdProcessSuccessState(
          response: response.data!,
        ));
      } else {
        emit(SoliqIdErrorState(
          error: response.responseMessage!,
        ));
      }
    } on Object catch (e) {
      log(e.toString());
    }
  }

  FutureOr<void> _onSetCompareFace(
      SoliqIdCompareFaceEvent event, Emitter<SoliqIdState> emit) async {
    try {
      // final response = await repository.compareFace(
      //   id: event.id,
      //   asked: event.asked,
      //   processId: event.processId,
      //   image: event.date,
      // );
      // emit(SoliqIdCompareFaceSuccessState(
      //   response: response,
      // ));

      emit(SoliqIdCompareFaceLoadingState());

      final response = await idRepository.compareFace(
        id: event.id,
        asked: event.asked,
        processId: event.processId,
        image: event.date,
      );

      if (response.status && response.data != null) {
        emit(SoliqIdCompareFaceSuccessState(
          response: response.data!,
        ));
      } else {
        emit(SoliqIdErrorState(
          error: response.responseMessage!,
        ));
      }
    } on Object catch (e) {
      log(e.toString());
    }
  }
}
