import 'dart:developer';

import 'package:soliq_id/src/data/dio/network_response_extension.dart';
import 'package:soliq_id/src/data/dio/request_service.dart';
import 'package:soliq_id/src/data/dio/type_parse_service.dart';
import 'package:soliq_id/src/data/model/response/account_info_model.dart';
import 'package:soliq_id/src/data/model/response/compare_face_response.dart';
import 'package:soliq_id/src/data/model/response/create_session.dart';
import 'package:soliq_id/src/data/model/response/device_model.dart';
import 'package:soliq_id/src/data/model/response/process_response_model.dart';
import 'package:soliq_id/src/data/model/response/response_model.dart';

class IdNetworkService {
  final NetworkService networkService;

  IdNetworkService({required this.networkService});

  Future<DataResponseModel<AccountInfoModel>> getAccountInfo({
    required DeviceModel device,
    required int pinfl,
  }) async {

    log("Hi");
    // try {
      final response = await networkService.postMethod(
        url: "soliq-id/get-account-info",
        body: {
          "device": device.toJson(),
          "pinfl": pinfl,
        },
      );
      if (response.status) {
        if (response.isStatus) {
          return DataResponseModel.success(
              model: AccountInfoModel.fromJson(response.response!.data));
        } else {
          return DataResponseModel.error(
              responseMessage:
                  parseToString(response.response?.data, "message"));
        }
      } else {
        return DataResponseModel.error(
            responseMessage: response.responseMessage);
      }
    // } catch (e) {
    //   return DataResponseModel.error(responseMessage: e.toString());
    // }
  }

  Future<DataResponseModel<CreateSession>> session({
    required DeviceModel device,
    required int pinfl,
    required String fio,
  }) async {
    try {
      final response = await networkService.postMethod(
        url: "soliq-id/create-session",
        body: {
          "device": device.toJson(),
          "pinfl": pinfl,
          "fio": fio,
        },
      );
      if (response.status) {
          return DataResponseModel.success(
              model: CreateSession.fromMap(response.response!.data));

      } else {
        return DataResponseModel.error(
            responseMessage: response.responseMessage);
      }
    } catch (e) {
      return DataResponseModel.error(responseMessage: e.toString());
    }
  }

  Future<DataResponseModel<ProcessResponseModel>> logsApi({
    required String deviceId,
    required String processId,
    required int id,
    required int asked,
    required String date,
  }) async{
    try{
      final response = await networkService.postMethod(
        url: "soliq-id/set-process",
        body: {
          "deviceId": deviceId,
          "processId": processId,
          "id": id,
          "asked": asked,
          "date": date,
        },
      );
      if(response.status){
        if(response.isStatus){
          return DataResponseModel.success(
            model: ProcessResponseModel.fromJson(response.response!.data),
          );
        }else{
          return DataResponseModel.error(
            responseMessage: parseToString(response.response?.data, "message"),
          );
        }
      }else{
        return DataResponseModel.error(
          responseMessage: response.responseMessage,
        );
      }
    }catch(e){
      return DataResponseModel.error(responseMessage: e.toString());
    }
  }

  Future<DataResponseModel<CompareFaceResponse>> faceCompare({
    required String deviceId,
    required String processId,
    required int id,
    required int asked,
    required String image,
  }) async {
    try {
      final response = await networkService.postMethod(
        url: "soliq-id/compare-face",
        body: {
          "deviceId": deviceId,
          "processId": processId,
          "id": id,
          "asked": asked,
          "image": image,
        },
      );
      if (response.status) {
        if (response.isStatus) {
          return DataResponseModel.success(
              model: CompareFaceResponse.fromJson(response.response!.data));
        } else {
          return DataResponseModel.error(
              responseMessage:
                  parseToString(response.response?.data, "message"));
        }
      } else {
        return DataResponseModel.error(
            responseMessage: response.responseMessage);
      }
    } catch (e) {
      return DataResponseModel.error(responseMessage: e.toString());
    }
  }
}
