import 'package:soliq_id/src/data/dio/id_network_service.dart';
import 'package:soliq_id/src/data/model/response/account_info_model.dart';
import 'package:soliq_id/src/data/model/response/compare_face_response.dart';
import 'package:soliq_id/src/data/model/response/create_session.dart';
import 'package:soliq_id/src/data/model/response/process_response_model.dart';
import 'package:soliq_id/src/data/model/response/response_model.dart';
import 'package:soliq_id/src/utils/utils.dart';

class IdRepository {
  final IdNetworkService idNetworkService;

  IdRepository({required this.idNetworkService});

  Future<DataResponseModel<AccountInfoModel>> setAccountInfo({
    required int pinfl,
  }) async {
    final device = await Utils.getDeviceDetails();
    return await idNetworkService.getAccountInfo(
      device: device,
      pinfl: pinfl,
    );
  }

  Future<DataResponseModel<CreateSession>> createSession({
    required int pinfl,
    required String fio,
  }) async {
    final device = await Utils.getDeviceDetails();
    return await idNetworkService.session(
      device: device,
      pinfl: pinfl,
      fio: fio,
    );
  }

  Future<DataResponseModel<CompareFaceResponse>> compareFace({
    required int id,
    required int asked,
    required String processId,
    required String image,
  }) async {
    final deviceId = await Utils.getDeviceId();
    return await idNetworkService.faceCompare(
      deviceId: deviceId,
      processId: processId,
      id: id,
      asked: asked,
      image: image,
    );
  }


  /// Set process
  Future<DataResponseModel<ProcessResponseModel>> setProcess({
    required String processId,
    required int id,
    required int asked,
    required String date,
  }) async {
    final deviceId = await Utils.getDeviceId();
    return await idNetworkService.logsApi(
      deviceId: deviceId,
      processId: processId,
      id: id,
      asked: asked,
      date: date,
    );
  }
}
