import 'package:soliq_id/src/data/dio/type_parse_service.dart';

class CompareFaceResponse {
  final bool success;
  final int code;
  final String message;
  final int id;
  final String token;
  final CompareFaceData data;

  CompareFaceResponse({
    required this.success,
    required this.code,
    required this.message,
    required this.id,
    required this.token,
    required this.data,
  });

  factory CompareFaceResponse.fromJson(Map<String, dynamic> response) {
    return CompareFaceResponse(
      success: parseToBool(response, "success"),
      code: parseToInt(response, "code"),
      message: parseToString(response, "message"),
      id: parseToInt(response, "id"),
      token: parseToString(response, "token"),
      data: response.containsKey("data") && response["data"] != null
          ? CompareFaceData.fromJson(response["data"])
          : CompareFaceData.example(),
    );
  }

  Map<String, dynamic> toJson() {
    return {
      "success": success,
      "code": code,
      "message": message,
      "id": id,
      "token": token,
      "data": data.toJson(),
    };
  }
}

class CompareFaceData {
  final String pinfl;
  final String sureName;
  final String name;
  final String fatherName;
  final String birthDate;
  final String birthPlace;
  final String docSerial;
  final String docNumber;
  final String livingPlace;
  final String docGivenDate;
  final String docExpireDate;
  final String docGivenPlace;
  final int status;

  CompareFaceData({
    required this.pinfl,
    required this.sureName,
    required this.name,
    required this.fatherName,
    required this.birthDate,
    required this.birthPlace,
    required this.docSerial,
    required this.docNumber,
    required this.livingPlace,
    required this.docGivenDate,
    required this.docExpireDate,
    required this.docGivenPlace,
    required this.status,
  });

  factory CompareFaceData.fromJson(Map<String, dynamic> response) {
    return CompareFaceData(
      pinfl: parseToString(response, "pinfl"),
      sureName: parseToString(response, "surName"),
      name: parseToString(response, "name"),
      fatherName: parseToString(response, "patronymicName"),
      birthDate: parseToString(response, "bithDate"),
      birthPlace: parseToString(response, "bithPlace"),
      docSerial: parseToString(response, "docSeria"),
      docNumber: parseToString(response, "docNumber"),
      livingPlace: parseToString(response, "livingPlace"),
      docGivenDate: parseToString(response, "docGiveDate"),
      docExpireDate: parseToString(response, "docExpirationDate"),
      docGivenPlace: parseToString(response, "docGivePlace"),
      status: parseToInt(response, "status"),
    );
  }

  Map<String, dynamic> toJson() {
    return {
      "pinfl": pinfl,
      "surName": sureName,
      "name": name,
      "patronymicName": fatherName,
      "birthDate": birthDate,
      "birthPlace": birthPlace,
      "docSerial": docSerial,
      "docNumber": docNumber,
      "livingPlace": livingPlace,
      "docGiveDate": docGivenDate,
      "docExpirationDate": docExpireDate,
      "docGivePlace": docGivenPlace,
      "status": status,
    };
  }

  factory CompareFaceData.example() {
    return CompareFaceData(
      pinfl: "123456789012",
      sureName: "SureName",
      name: "Name",
      fatherName: "FatherName",
      birthDate: "01.01.2000",
      birthPlace: "BirthPlace",
      docSerial: "AA",
      docNumber: "1234567",
      livingPlace: "LivingPlace",
      docGivenDate: "01.01.2000",
      docExpireDate: "01.01.2020",
      docGivenPlace: "DocGivenPlace",
      status: 1,
    );
  }
}
