import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:soliq_id/src/data/dio/id_network_service.dart';
import 'package:soliq_id/src/data/dio/id_repository.dart';
import 'package:soliq_id/src/data/dio/request_service.dart';
import 'package:soliq_id/src/screens/passive_camera_screen2.dart';
import 'package:soliq_id/src/screens/widgets/message_screen.dart';
import 'package:soliq_id/src/utils/assets.dart';
import 'package:soliq_id/src/utils/navigator_extension.dart';

import '../../localizations_util.dart';
import '../data/bloc/soliq_id_bloc.dart';
import '../utils/color_utils.dart';
import '../utils/typography.dart';

LocalizationType localeEnum = LocalizationType.UZ;

class FaceIdSignIn extends StatefulWidget {
  final String pinfl;
  final LocalizationType localizationType;
  final String guid;
  final Function onEvent;
  final bool onNavigate;

  const FaceIdSignIn({
    Key? key,
    required this.pinfl,
    required this.localizationType,
    required this.guid,
    required this.onEvent,
    required this.onNavigate,
  }) : super(key: key);

  @override
  State<FaceIdSignIn> createState() => _FaceIdSignInState();
}

class _FaceIdSignInState extends State<FaceIdSignIn> {
  String? res;
  String buttonTitle = '';
  final DateTime date = DateTime.now();

  @override
  void initState() {
    super.initState();

    localeEnum = widget.localizationType;
    BlocProvider.of<SoliqIdBloc>(context).add(
      SoliqIdAccountInfoEvent(pinfl: int.parse(widget.pinfl), lang: "uz"),
    );
  }

  @override
  Widget build(BuildContext context) {
    buttonTitle = getString('check');

    return Scaffold(
      backgroundColor: AppColor.appColor,
      appBar: AppBar(
        backgroundColor: AppColor.appColor,
        title: const Text(
          "YuzID",
          style: TextStyle(
            fontSize: 16,
            fontWeight: FontWeight.w500,
            color: Colors.black,
          ),
        ),
        centerTitle: true,
        elevation: 0,
      ),
      body: BlocConsumer<SoliqIdBloc, SoliqIdState>(
        listener: (context, state) async {
          if (state is SoliqIdAccountInfoSuccessState) {
            BlocProvider.of<SoliqIdBloc>(context).add(SoliqIdCreateSessionEvent(
              fio: state.response.data.fio,
              pinfl: int.parse(widget.pinfl),
              lang: "uz",
            ));
          }
          if (state is SoliqIdErrorState) {
            ScaffoldMessenger.of(context).showSnackBar(
              SnackBar(
                content: Text(state.error),
              ),
            );
          }

          if (state is SoliqIdCompareFaceSuccessState &&
              state.response.success == false) {
            if (widget.onNavigate == true) {
              await Navigator.push(
                context,
                MaterialPageRoute(
                  builder: (context) => MessageScreen(
                    compareFaceResponse: state.response,
                  ),
                ),
              );
            }

            widget.onEvent.call(state.response.toJson());

            Navigator.pop(context);
          }
          if (state is SoliqIdCompareFaceSuccessState &&
              state.response.success == true) {
            if (widget.onNavigate == true) {
              await Navigator.push(
                context,
                MaterialPageRoute(
                  builder: (_) => MessageScreen(
                    compareFaceResponse: state.response,
                  ),
                ),
              );
            }

            widget.onEvent.call(state.response.toJson());
            Navigator.pop(context);
          }
        },
        builder: (context, state) {
          return Stack(
            children: [
              Align(
                alignment: Alignment.topCenter,
                child: Column(
                  children: [
                    const SizedBox(
                      height: 8,
                    ),
                    SizedBox(
                      width: MediaQuery.of(context).size.width / 2,
                      height: MediaQuery.of(context).size.width / 2,
                      child: Container(
                          margin: const EdgeInsets.all(18),
                          child: Image.asset(Assets.face_icon)),
                    ),
                    const SizedBox(
                      height: 24,
                    ),
                    BlocBuilder<SoliqIdBloc, SoliqIdState>(
                      buildWhen: (p, c) => c is SoliqIdAccountInfoSuccessState,
                      builder: (context, state) {
                        if (state is SoliqIdAccountInfoSuccessState) {
                          return Text(
                            state.response.data.fio,
                            style: const TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w700,
                            ),
                          );
                        }
                        return const Text(
                          "",
                          style: TextStyle(
                            fontSize: 16,
                            fontWeight: FontWeight.w700,
                          ),
                        );
                      },
                    ),
                    const SizedBox(
                      height: 6,
                    ),
                    Text(
                      "${getString('pinfl')}: ${widget.pinfl}",
                      style: const TextStyle(
                        fontSize: 16,
                        fontWeight: FontWeight.w700,
                      ),
                    ),
                    const SizedBox(
                      height: 26,
                    ),
                    Container(
                      margin: const EdgeInsets.symmetric(horizontal: 28),
                      child: Text(getString('suggest_advice'),
                          textAlign: TextAlign.start,
                          style: FTypography.h1TextBold),
                    ),
                    const SizedBox(
                      height: 18,
                    ),
                    Padding(
                      padding: const EdgeInsets.symmetric(
                        horizontal: 24,
                      ),
                      child: Text(
                        getString("advice"),
                        textAlign: TextAlign.start,
                        style: FTypography.h1Text1,
                      ),
                    ),
                    const SizedBox(
                      height: 18,
                    ),
                  ],
                ),
              ),
              Positioned(
                bottom: 0,
                left: 0,
                right: 0,
                child: Container(
                  margin: const EdgeInsets.only(bottom: 25),
                  child: Column(
                    children: [
                      BlocBuilder<SoliqIdBloc, SoliqIdState>(
                        builder: (context, state) {
                          log("State $state");
                          return InkWell(
                            onTap: (state is SoliqIdPinflLoadingState ||
                                    state is SoliqIdCompareFaceLoadingState || state is SoliqIdErrorState)
                                ? () {}
                                : () async {
                                    if (state
                                        is SoliqIdCreateSessionSuccessState) {
                                      final response = await pushWithBloc<
                                              SoliqIdBloc>(
                                          // CameraScreen(session: state.response,),
                                          const PassiveCameraScreen(),
                                          SoliqIdBloc(IdRepository(
                                              idNetworkService:
                                                  IdNetworkService(
                                                      networkService:
                                                          NetworkService()))));

                                      if (response != null) {
                                        BlocProvider.of<SoliqIdBloc>(context).add(
                                    SoliqIdCompareFaceEvent(
                                      processId: state.response.sessionId,
                                      id: -60,
                                      asked: 7,
                                      date: response,
                                    ),
                                  );
                                }
                              }
                                  },
                            child: Container(
                              margin:
                                  const EdgeInsets.symmetric(horizontal: 28),
                              height: 56,
                              width: MediaQuery.of(context).size.width,
                              decoration: BoxDecoration(
                                  color: state is SoliqIdPinflLoadingState ||
                                          state
                                              is SoliqIdCompareFaceLoadingState ||   state is SoliqIdErrorState
                                      ? Colors.grey
                                      : AppColor.custom_redColor,
                                  borderRadius: BorderRadius.circular(12)),
                              child: Stack(
                                children: [
                                  Center(
                                    child: state is SoliqIdPinflLoadingState ||
                                            state
                                                is SoliqIdCompareFaceLoadingState
                                        ? Container(
                                            margin: const EdgeInsets.only(
                                                right: 12),
                                            width: 24,
                                            height: 24,
                                            child:
                                                const CircularProgressIndicator())
                                        : Text(
                                            buttonTitle,
                                            style: const TextStyle(
                                                fontWeight: FontWeight.w500,
                                                fontSize: 16,
                                                color: Colors.white),
                                          ),
                                  ),
                                ],
                              ),
                            ),
                          );
                        },
                      ),
                    ],
                  ),
                ),
              ),
            ],
          );
        },
      ),
    );
  }


}