import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import '../../utils/color_utils.dart';

class CustomTextFormField extends StatelessWidget {
  const CustomTextFormField({
    Key? key,
    required this.controller,
    required this.prefix,
    required this.suffix,
    required this.hintText,
    required this.textInputType,
    this.onChanged,
    this.focusNode,
    this.maxLength,
    this.inputFormatters,
    this.errorText, this.errorColors,
  }) : super(key: key);
  final TextEditingController controller;
  final Widget prefix;
  final Widget suffix;
  final String hintText;
  final int? maxLength;
  final FocusNode? focusNode;
  final List<TextInputFormatter>? inputFormatters;
  final TextInputType textInputType;
  final onChanged;
  final String? errorText;
  final Color? errorColors;
  @override
  Widget build(BuildContext context) {
    return Container(
        margin: const EdgeInsets.symmetric(
          horizontal: 28,
        ),
        padding: const EdgeInsets.symmetric(vertical: 10),
        child: TextFormField(
          controller: controller,
          keyboardType: textInputType,
          focusNode: focusNode,
          maxLength: maxLength,
          inputFormatters: inputFormatters,
          style: const TextStyle(
            fontSize: 18,
            fontWeight: FontWeight.w500,
            color: Colors.black,
          ),
          decoration: InputDecoration(
            contentPadding: const EdgeInsets.symmetric(vertical: 10),
            errorText: errorText,
            errorBorder: const OutlineInputBorder(
              borderSide: BorderSide(
                width: 1,
                color: AppColor.custom_redColor,
              ),
              borderRadius: BorderRadius.all(
                Radius.circular(12),
              ),
            ),
            filled: true,
            fillColor: const Color(0xFFFFFFFF),
            border: const OutlineInputBorder(
              borderSide: BorderSide.none,
              borderRadius: BorderRadius.all(
                Radius.circular(12),
              ),
            ),
            suffixIcon: suffix,
            prefixIcon: prefix,
            hintStyle: const TextStyle(
              fontSize: 18,
              color: Color(0xFF949494),
            ),
            errorStyle: TextStyle(
                fontSize: 16,
                fontWeight: FontWeight.w500,
                color: errorColors!,

            ),
            hintText: hintText,
          ),
          onChanged: onChanged,
        ),
    );
  }
}
